from steamship_tests import PACKAGES_PATH
from steamship_tests.utils.deployables import deploy_package
from steamship_tests.utils.fixtures import get_steamship_client


def test_adding_configuration():
    """NOTE: Any changes to the package under test need to be reflected in the paired documentation pages!"""

    client = get_steamship_client()
    demo_package_path = PACKAGES_PATH / "adding_configuration.py"
    config_template = {
        "name": {"type": "string"},
        "favoriteColor": {"type": "string"},
        "luckyNumber": {"type": "number"},
        "favoriteTrueFalseValue": {"type": "boolean"},
    }
    instance_config = {
        "name": "Shaggy",
        "favoriteColor": "avocado green",
        "luckyNumber": 42,
        "favoriteTrueFalseValue": True,
    }

    with deploy_package(
        client,
        demo_package_path,
        version_config_template=config_template,
        instance_config=instance_config,
    ) as (_, _, instance):
        response = instance.invoke("my_faves")
        assert response is not None
        assert (
            response
            == """
        Hey Shaggy!
        I can remind you of your favorites.
        Your favorite color is avocado green.
        Your lucky number is 42.0.
        Your favorite true/false value is True.
        Wow, mine too!
        """
        )



import pytest

from steamship import Steamship
from steamship.agents.examples.document_qa_agent import ExampleDocumentQAService
from steamship.utils.url import Verb


@pytest.mark.usefixtures("client")
def test_indexer_pipeline_mixin(client: Steamship):
    package_class = ExampleDocumentQAService

    post_routes = package_class._package_spec.method_mappings[Verb.POST]
    assert "/index_text" in post_routes  # from indexer mixin
    assert "/index_url" in post_routes  # From indexer pipeline mixin
    assert "/blockify_file" in post_routes  # From blockifier mixin
    assert "/import_url" in post_routes  # from file importer mixin



from steamship.invocable import Config, InvocableResponse, InvocationContext, PackageService, post
from steamship.invocable.package_mixin import PackageMixin


class TestMixin(PackageMixin):

    suffix: str
    instance_init_called: bool = False

    def __init__(self, suffix: str):
        self.suffix = suffix

    def instance_init(self, config: Config, context: InvocationContext):
        self.instance_init_called = True

    @post("test_mixin_route", public=True)
    def test_mixin_route(self, text: str) -> InvocableResponse:
        _ = text
        return InvocableResponse(data=f"mixin {self.suffix}")


class PackageWithMixin(PackageService):

    USED_MIXIN_CLASSES = [TestMixin]

    def __init__(self, **kwargs):
        super(PackageWithMixin, self).__init__(**kwargs)
        self.add_mixin(
            TestMixin("yo")
        )  # We add instances of mixins, not the class, in case we need to parameterize their inits

    @post("test_package_route")
    def test_package_route(self) -> InvocableResponse:
        return InvocableResponse(data="package")



import uuid
from typing import List

from steamship import Block
from steamship.agents.llms.openai import OpenAI
from steamship.agents.react import ReACTAgent
from steamship.agents.schema import AgentContext
from steamship.agents.schema.context import Metadata
from steamship.agents.service.agent_service import AgentService
from steamship.agents.tools.question_answering import VectorSearchQATool
from steamship.invocable import post
from steamship.invocable.mixins.blockifier_mixin import BlockifierMixin
from steamship.invocable.mixins.file_importer_mixin import FileImporterMixin
from steamship.invocable.mixins.indexer_mixin import IndexerMixin
from steamship.invocable.mixins.indexer_pipeline_mixin import IndexerPipelineMixin
from steamship.utils.repl import AgentREPL


class ExampleDocumentQAService(AgentService):
    """DocumentQAService is an example AgentService that exposes:  # noqa: RST201

    - A few authenticated endpoints for learning PDF and YouTube documents:

         /learn_url
        { url }

        /learn_text
        { text }

    - An unauthenticated endpoint for answering questions about what it has learned

    This agent provides a starter project for special purpose QA agents that can answer questions about documents
    you provide.
    """

    USED_MIXIN_CLASSES = [IndexerPipelineMixin, FileImporterMixin, BlockifierMixin, IndexerMixin]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # This Mixin provides HTTP endpoints that coordinate the learning of documents.
        #
        # It adds the `/learn_url` endpoint which will:
        #    1) Download the provided URL (PDF, YouTube URL, etc)
        #    2) Convert that URL into text
        #    3) Store the text in a vector index
        #
        # That vector index is then available to the question answering tool, below.
        self.add_mixin(IndexerPipelineMixin(self.client, self))

        # A ReACTAgent is an agent that is able to:
        #    1) Converse with you, casually... but also
        #    2) Use tools that have been provided to it, such as QA tools or Image Generation tools
        #
        # This particular ReACTAgent has been provided with a single tool which will be used whenever
        # the user answers a question. But you can extend this with more tools if you wish. For example,
        # you could add tools to generate images, or search Google, or register an account.
        self._agent = ReACTAgent(
            tools=[
                VectorSearchQATool(),  # Tool to answer questions based on a vector store.
            ],
            llm=OpenAI(self.client),
        )

    @post("prompt")
    def prompt(self, prompt: str) -> str:
        """Run an agent with the provided text as the input."""

        # AgentContexts serve to allow the AgentService to run agents
        # with appropriate information about the desired tasking.
        # Here, we create a new context on each prompt, and append the
        # prompt to the message history stored in the context.
        context_id = uuid.uuid4()
        context = AgentContext.get_or_create(self.client, {"id": f"{context_id}"})
        context.chat_history.append_user_message(prompt)

        # AgentServices provide an emit function hook to access the output of running
        # agents and tools. The emit functions fire at after the supplied agent emits
        # a "FinishAction".
        #
        # Here, we show one way of accessing the output in a synchronous fashion. An
        # alternative way would be to access the final Action in the `context.completed_steps`
        # after the call to `run_agent()`.
        output = ""

        def sync_emit(blocks: List[Block], meta: Metadata):
            nonlocal output
            block_text = "\n".join(
                [b.text if b.is_text() else f"({b.mime_type}: {b.id})" for b in blocks]
            )
            output += block_text

        context.emit_funcs.append(sync_emit)
        self.run_agent(self._agent, context)
        return output


if __name__ == "__main__":
    # AgentREPL provides a mechanism for local execution of an AgentService method.
    # This is used for simplified debugging as agents and tools are developed and
    # added.
    AgentREPL(ExampleDocumentQAService, "prompt", agent_package_config={}).run()



"""Tool for generating images."""
from enum import Enum
from typing import Any, List, Optional, Union

from pydantic import BaseModel, Field

from steamship import Block, Steamship, Task
from steamship.agents.llms import OpenAI
from steamship.agents.schema import AgentContext
from steamship.agents.tools import VideoGeneratorTool
from steamship.agents.utils import with_llm
from steamship.utils.repl import ToolREPL

DEFAULT_SOURCE_URL = "https://www.steamship.com/images/agents/man-in-suit-midjourney.png"


class DIDVideoGeneratorTool(VideoGeneratorTool):
    """Tool to generate talking avatars from text using D-ID."""

    class Expression(BaseModel):
        class Expressions(str, Enum):
            NEUTRAL = "neutral"
            HAPPY = "happy"
            SURPRISE = "surprise"
            SERIOUS = "serious"

        start_frame: int = Field(ge=0)
        expression: Expressions
        intensity: float = Field(1.0, le=1, ge=0)

    name: str = "DIDVideoGeneratorTool"
    human_description: str = "Generates an a video of you speaking a response to a user."
    agent_description = (
        "Used to generate a video of you from text. Use if the user has asked for a video response.  "
        "The input is the text that you want to say. "
        "The output is the video of you saying it."
    )
    generator_plugin_handle: str = "did-video-generator"
    generator_plugin_config: dict = {}

    source_url: Optional[str] = DEFAULT_SOURCE_URL
    """The URL of the source image to be animated."""

    stitch: bool = True

    voice_provider: Optional[str] = "microsoft"
    """The voice provider. Must be either `microsoft` or `amazon`."""

    voice_id: Optional[str] = "en-US-GuyNeural"
    """The voice ID. E.g. `en-US-AshleyNeural` for Microsoft or `Amy` for Amazon."""

    voice_style: Optional[str] = "Default"

    driver_url: Optional[str] = None
    """The URL of the D-ID driver video. If not provided a driver video will be selected automatically."""

    expressions: Optional[List[Expression]] = None
    """A list of expressions to apply."""

    transition_frames: Optional[int] = 20
    """How many frames to use for expression transition."""

    def run(self, tool_input: List[Block], context: AgentContext) -> Union[List[Block], Task[Any]]:
        """Run the tool. Copied from base class to enable generate-time config overrides."""

        generator = context.client.use_plugin(
            plugin_handle=self.generator_plugin_handle,
            instance_handle=self.generator_plugin_instance_handle,
            config=self.generator_plugin_config,
        )

        tasks = []

        if self.merge_blocks:
            block = tool_input[0]
            for extra_block in tool_input[1:]:
                block.text = f"{block.text}\n\n{extra_block.text}"
            tool_input = [block]

        for block in tool_input:
            if not block.is_text():
                continue

            prompt = block.text
            task = generator.generate(
                text=prompt,
                make_output_public=True,
                append_output_to_file=True,
                options={
                    "source_url": self.source_url,
                    "stitch": self.stitch,
                    "provider": {
                        "type": self.voice_provider,
                        "voice_id": self.voice_id,
                        "voice_config": {"style": self.voice_style},
                        "expressions": self.expressions,
                    },
                    "transition_frames": self.transition_frames,
                },
            )
            tasks.append(task)

        # TODO / REMOVE Synchronous execution is a temporary simplification while we merge code.
        output_blocks = []
        # NB: In an async framework, we need to contend with the fact that we have some Monad-style trickery
        # to contend with. Namely that the below code has taken us from List[Block] as the universal type to
        # List[List[Block]] as the async result that we will have to process.
        #
        # It's unclear to me (ted) if this is something we leave as an exercise for implementors or build in
        # as universally handled.
        for task in tasks:
            task.wait()
            task_blocks = self.post_process(task, context)
            for block in task_blocks:
                output_blocks.append(block)
        # END TODO / REMOVE

        return output_blocks


if __name__ == "__main__":
    tool = DIDVideoGeneratorTool(expressions=[{"start_frame": 0, "expression": "happy"}])
    with Steamship.temporary_workspace() as client:
        ToolREPL(tool).run_with_client(client=client, context=with_llm(llm=OpenAI(client=client)))



from typing import Dict

import pytest
import requests
from assets.packages.transports.mock_telegram_package import MockTelegram
from steamship_tests import PACKAGES_PATH
from steamship_tests.utils.deployables import deploy_package

from steamship import File, Steamship

config_template = {
    "bot_token": {"type": "string"},
    "api_base": {"type": "string"},
}


@pytest.mark.usefixtures("client")
def test_telegram(client: Steamship):

    mock_telegram_package_path = PACKAGES_PATH / "transports" / "mock_telegram_package.py"
    telegram_agent_path = PACKAGES_PATH / "transports" / "test_telegram_agent.py"

    with deploy_package(client, mock_telegram_package_path) as (_, _, mock_telegram_instance):

        # Removing the / from the invocation URL and passing it as the bot_token allows
        # the bot token to not be empty and the two appended to each other to just equal the
        # invocation url.
        instance_config = {"bot_token": "/", "api_base": mock_telegram_instance.invocation_url[:-1]}

        # Should be able to invoke setWebhook publicly
        response = requests.get(
            url=mock_telegram_instance.invocation_url + "/setWebhook",
            params={"url": "test", "allowed_updates": "", "drop_pending_updates": True},
        )
        assert response.ok
        # Test that the call to setWebhook wrote a file
        files = File.query(client, f'kind "{MockTelegram.WEBHOOK_TAG}"').files
        assert len(files) == 1
        assert files[0].tags[0].name == "test"

        files[0].delete()

        with deploy_package(
            client,
            telegram_agent_path,
            version_config_template=config_template,
            instance_config=instance_config,
        ) as (_, _, telegram_instance):

            # Test that agent called instance_init and registered webhook
            files = File.query(client, f'kind "{MockTelegram.WEBHOOK_TAG}"').files
            assert len(files) == 1
            assert files[0].tags[0].name == telegram_instance.invocation_url + "telegram_respond"

            # test sending messages (without auth)
            response = requests.post(
                url=telegram_instance.invocation_url + "/telegram_respond",
                json=generate_telegram_message("a test"),
            )
            assert response.ok
            files = File.query(client, f'kind "{MockTelegram.TEXT_MESSAGE_TAG}"').files
            assert len(files) == 1
            assert files[0].tags[0].name == "Response to: a test".replace(
                " ", "+"
            )  # bug somewhere - results being url encoded
            assert files[0].tags[0].value == {MockTelegram.CHAT_ID_KEY: 1}

            # test sending another message; this has been a problem before
            telegram_instance.invoke(
                "telegram_respond", **generate_telegram_message("another test")
            )
            files = File.query(client, f'kind "{MockTelegram.TEXT_MESSAGE_TAG}"').files
            assert len(files) == 2
            for file in files:
                assert file.tags[0].name in [
                    "Response to: a test".replace(" ", "+"),
                    "Response to: another test".replace(" ", "+"),
                ]  # bug somewhere - results being url encoded
                assert file.tags[0].value == {MockTelegram.CHAT_ID_KEY: 1}

            # Test the agent sending a "photo"
            telegram_instance.invoke("telegram_respond", **generate_telegram_message("image"))
            files = File.query(client, f'kind "{MockTelegram.PHOTO_MESSAGE_TAG}"').files
            assert len(files) == 1
            assert files[0].tags[0].value == {
                MockTelegram.CHAT_ID_KEY: 1,
                MockTelegram.PHOTO_KEY: "some image bytes",
            }

            # Test the agent sending "audio"
            telegram_instance.invoke("telegram_respond", **generate_telegram_message("audio"))
            files = File.query(client, f'kind "{MockTelegram.AUDIO_MESSAGE_TAG}"').files
            assert len(files) == 1
            assert files[0].tags[0].value == {
                MockTelegram.CHAT_ID_KEY: 1,
                MockTelegram.AUDIO_KEY: "some audio bytes",
            }

            # Test the agent sending a "video"
            telegram_instance.invoke("telegram_respond", **generate_telegram_message("video"))
            files = File.query(client, f'kind "{MockTelegram.VIDEO_MESSAGE_TAG}"').files
            assert len(files) == 1
            assert files[0].tags[0].value == {
                MockTelegram.CHAT_ID_KEY: 1,
                MockTelegram.VIDEO_KEY: "some video bytes",
            }


def generate_telegram_message(text: str) -> Dict:
    return {
        "update_id": 1,
        "message": {"message_id": 1, "chat": {"id": 1}, "text": text},
    }



import logging
import tempfile
from typing import Any, Dict, List, Optional

import requests
from pydantic import Field

from steamship import Block, Steamship, SteamshipError
from steamship.agents.llms import OpenAI
from steamship.agents.mixins.transports.transport import Transport
from steamship.agents.schema import Agent, AgentContext, EmitFunc, Metadata
from steamship.agents.service.agent_service import AgentService
from steamship.agents.utils import with_llm
from steamship.invocable import Config, InvocableResponse, InvocationContext, post


class TelegramTransportConfig(Config):
    bot_token: str = Field(description="The secret token for your Telegram bot")
    api_base: str = Field("https://api.telegram.org/bot", description="The root API for Telegram")


class TelegramTransport(Transport):
    """Experimental base class to encapsulate a Telegram communication channel."""

    api_root: str
    bot_token: str
    agent: Agent
    agent_service: AgentService

    def __init__(
        self,
        client: Steamship,
        config: TelegramTransportConfig,
        agent_service: AgentService,
        agent: Agent,
    ):
        super().__init__(client=client)
        self.api_root = f"{config.api_base}{config.bot_token}"
        self.bot_token = config.bot_token
        self.agent = agent
        self.agent_service = agent_service

    def instance_init(self, config: Config, invocation_context: InvocationContext):
        webhook_url = invocation_context.invocable_url + "telegram_respond"

        logging.info(
            f"Setting Telegram webhook URL: {webhook_url}. Post is to {self.api_root}/setWebhook"
        )

        response = requests.get(
            f"{self.api_root}/setWebhook",
            params={
                "url": webhook_url,
                "allowed_updates": ["message"],
                "drop_pending_updates": True,
            },
        )

        if not response.ok:
            raise SteamshipError(
                f"Could not set webhook for bot. Webhook URL was {webhook_url}. Telegram response message: {response.text}"
            )

    @post("telegram_webhook_info")
    def telegram_webhook_info(self) -> dict:
        return requests.get(self.api_root + "/getWebhookInfo").json()

    @post("telegram_disconnect_webhook")
    def telegram_disconnect_webhook(self, *args, **kwargs):
        """Unsubscribe from Telegram updates."""
        requests.get(f"{self.api_root}/deleteWebhook")

    def _send(self, blocks: [Block], metadata: Metadata):
        """Send a response to the Telegram chat."""
        for block in blocks:
            chat_id = block.chat_id
            if block.is_text() or block.text:
                params = {"chat_id": int(chat_id), "text": block.text}
                requests.get(f"{self.api_root}/sendMessage", params=params)
            elif block.is_image() or block.is_audio() or block.is_video():
                if block.is_image():
                    suffix = "sendPhoto"
                    key = "photo"
                elif block.is_audio():
                    suffix = "sendAudio"
                    key = "audio"
                elif block.is_video():
                    suffix = "sendVideo"
                    key = "video"

                _bytes = block.raw()
                with tempfile.TemporaryFile(mode="r+b") as temp_file:
                    temp_file.write(_bytes)
                    temp_file.seek(0)
                    resp = requests.post(
                        url=f"{self.api_root}/{suffix}?chat_id={chat_id}",
                        files={key: temp_file},
                    )
                    if resp.status_code != 200:
                        logging.error(f"Error sending message: {resp.text} [{resp.status_code}]")
                        raise SteamshipError(
                            f"Message not sent to chat {chat_id} successfully: {resp.text}"
                        )
            else:
                logging.error(
                    f"Telegram transport unable to send a block of MimeType {block.mime_type}"
                )

    def _get_file(self, file_id: str) -> Dict[str, Any]:
        return requests.get(f"{self.api_root}/getFile", params={"file_id": file_id}).json()[
            "result"
        ]

    def _get_file_url(self, file_id: str) -> str:
        return f"https://api.telegram.org/file/bot{self.bot_token}/{self._get_file(file_id)['file_path']}"

    def _download_file(self, file_id: str):
        result = requests.get(self._get_file_url(file_id))
        if result.status_code != 200:
            raise Exception("Download file", result)

        return result.content

    def _parse_inbound(self, payload: dict, context: Optional[dict] = None) -> Optional[Block]:
        """Parses an inbound Telegram message."""

        chat = payload.get("chat")
        if chat is None:
            raise SteamshipError(f"No `chat` found in Telegram message: {payload}")

        chat_id = chat.get("id")
        if chat_id is None:
            raise SteamshipError(f"No 'chat_id' found in Telegram message: {payload}")

        if not isinstance(chat_id, int):
            raise SteamshipError(
                f"Bad 'chat_id' found in Telegram message: ({chat_id}). Should have been an int."
            )

        message_id = payload.get("message_id")
        if message_id is None:
            raise SteamshipError(f"No 'message_id' found in Telegram message: {payload}")

        if not isinstance(message_id, int):
            raise SteamshipError(
                f"Bad 'message_id' found in Telegram message: ({message_id}). Should have been an int"
            )

        if video_or_voice := (payload.get("voice") or payload.get("video_note")):
            file_id = video_or_voice.get("file_id")
            file_url = self._get_file_url(file_id)
            block = Block(
                text=payload.get("text"),
                url=file_url,
            )
            block.set_chat_id(str(chat_id))
            block.set_message_id(str(message_id))
            return block

        # Some incoming messages (like the group join message) don't have message text.
        # Rather than throw an error, we just don't return a Block.
        message_text = payload.get("text")
        if message_text is not None:
            result = Block(text=message_text)
            result.set_chat_id(str(chat_id))
            result.set_message_id(str(message_id))
            return result
        else:
            return None

    def build_emit_func(self, chat_id: str) -> EmitFunc:
        def new_emit_func(blocks: List[Block], metadata: Metadata):
            for block in blocks:
                block.set_chat_id(chat_id)
            return self.send(blocks, metadata)

        return new_emit_func

    @post("telegram_respond", public=True)
    def telegram_respond(self, **kwargs) -> InvocableResponse[str]:
        """Endpoint implementing the Telegram WebHook contract. This is a PUBLIC endpoint since Telegram cannot pass a Bearer token."""

        # TODO: must reject things not from the package
        message = kwargs.get("message", {})
        chat_id = message.get("chat", {}).get("id")
        try:
            incoming_message = self.parse_inbound(message)
            if incoming_message is not None:
                context = AgentContext.get_or_create(self.client, context_keys={"chat_id": chat_id})
                context.chat_history.append_user_message(
                    text=incoming_message.text, tags=incoming_message.tags
                )
                context.emit_funcs = [self.build_emit_func(chat_id=chat_id)]

                # Add an LLM to the context, using the Agent's if it exists.
                llm = None
                if hasattr(self.agent, "llm"):
                    llm = self.agent.llm
                else:
                    llm = OpenAI(client=self.client)

                context = with_llm(context=context, llm=llm)

                response = self.agent_service.run_agent(self.agent, context)
                if response is not None:
                    self.send(response)
                else:
                    # Do nothing here; this could be a message we intentionally don't want to respond to (ex. an image or file upload)
                    pass
            else:
                # Do nothing here; this could be a message we intentionally don't want to respond to (ex. an image or file upload)
                pass
        except Exception as e:
            response = self.response_for_exception(e, chat_id=chat_id)

            if chat_id is not None:
                self.send([response])
        # Even if we do nothing, make sure we return ok
        return InvocableResponse(string="OK")



import threading
from socketserver import TCPServer
from typing import Optional, Type

from steamship import Steamship
from steamship.cli.local_server.handler import create_safe_handler, make_handler
from steamship.invocable import (
    Invocable,
    InvocableRequest,
    Invocation,
    InvocationContext,
    LoggingConfig,
)


class SteamshipHTTPServer:
    """A simple HTTP Server that wraps an invocable (package or plugin).

    To use, call:

       server = SteamshipHTTPServer(invocable)
       server.start()

    To shut down, call:

       server.stop()

    """

    invocable: Type[Invocable]
    port: int
    server: TCPServer
    default_api_key: Optional[str] = (None,)
    invocable_handle: str = (None,)
    invocable_version_handle: str = (None,)
    invocable_instance_handle: str = (None,)
    add_port_to_invocable_url: bool = True

    def __init__(
        self,
        invocable: Type[Invocable],
        base_url: str = "http://localhost",
        port: int = 8080,
        default_api_key: Optional[str] = None,
        invocable_handle: str = None,
        invocable_version_handle: str = None,
        invocable_instance_handle: str = None,
        config: dict = None,
        add_port_to_invocable_url: bool = True,  # The invocable url represents the external URL, which may be NGROK
        workspace: str = None,
    ):
        self.invocable = invocable
        self.port = port
        self.base_url = base_url
        self.default_api_key = default_api_key
        self.invocable_handle = invocable_handle
        self.invocable_version_handle = invocable_version_handle
        self.invocable_instance_handle = invocable_instance_handle
        self.config = config
        self.add_port_to_invocable_url = add_port_to_invocable_url
        self.workspace = workspace

    def start(self):
        """Start the server."""

        handler = make_handler(
            self.invocable,
            self.port,
            base_url=self.base_url,
            default_api_key=self.default_api_key,
            invocable_handle=self.invocable_handle,
            invocable_version_handle=self.invocable_version_handle,
            invocable_instance_handle=self.invocable_instance_handle,
            config=self.config,
            add_port_to_invocable_url=self.add_port_to_invocable_url,
            workspace=self.workspace,
        )
        self.server = TCPServer(("", self.port), handler)
        # We need to trigger the instance init.
        context = InvocationContext(invocable_url=f"{self.base_url}/")

        invocation = Invocation(
            http_verb="POST", invocation_path="__dir__", arguments={}, config=self.config
        )
        event = InvocableRequest(
            client_config=Steamship(api_key=self.default_api_key, workspace=self.workspace).config,
            invocation=invocation,
            logging_config=LoggingConfig(logging_host=None, logging_port=None),
            invocation_context=context,
        )
        handler = create_safe_handler(self.invocable)
        handler(event.dict(by_alias=True), context)

        self.server.serve_forever()

    def stop(self):
        """Stop the server.

        Note: This has to be called from a different thread or else it will deadlock.
        """
        _server = self.server

        class ShutdownThread(threading.Thread):
            def __init__(self):
                threading.Thread.__init__(self)

            def run(self):
                nonlocal _server
                _server.shutdown()

        shutdown_thread = ShutdownThread()
        shutdown_thread.start()



from abc import ABC

from steamship.invocable import Config, InvocationContext


class PackageMixin(ABC):
    def instance_init(self, config: Config, context: InvocationContext):
        """The instance init method will be called ONCE by the engine when a new instance of a package or plugin has been created. By default, this does nothing."""
        pass



import pytest
from steamship_tests import PACKAGES_PATH
from steamship_tests.utils.deployables import deploy_package

from steamship import Steamship


@pytest.mark.usefixtures("client")
def test_mixin_and_package_invocation(client: Steamship):
    demo_package_path = PACKAGES_PATH / "package_with_mixins.py"

    with deploy_package(client, demo_package_path) as (_, _, instance):
        mixin_response = instance.invoke("test_mixin_route", text="test")
        assert mixin_response == "mixin yo"

        package_response = instance.invoke("test_package_route")
        assert package_response == "package"



import pytest

from steamship import Steamship
from steamship.agents.llms import OpenAI
from steamship.agents.schema import AgentContext
from steamship.agents.tools.question_answering import VectorSearchQATool
from steamship.agents.utils import with_llm
from steamship.invocable.mixins.indexer_mixin import IndexerMixin


@pytest.mark.usefixtures("client")
def test_indexer_mixin_and_qa_tool(client: Steamship):
    """Tests that we can inspect the package and mixin routes"""
    indexer = IndexerMixin(client)
    assert indexer.index_text("Mario was a very fun game.")
    res = indexer.search_index("Mario")
    assert res.items
    assert len(res.items) == 1
    item = res.items[0]
    assert item.tag.text == "Mario was a very fun game."

    tool = VectorSearchQATool()
    context = with_llm(
        OpenAI(client=client),
        AgentContext.get_or_create(client=client, context_keys={"id": "test"}),
    )
    res = tool.answer_question("Mario", context)
    assert len(res) == 1
    assert res[0].text  # Note we can't test for the exact value here since the LLM will weigh in.



from typing import List

from steamship import File, Tag
from steamship.invocable import PackageService, get, post


class MockTelegram(PackageService):
    WEBHOOK_TAG = "webhook"
    TEXT_MESSAGE_TAG = "text_message"
    CHAT_ID_KEY = "chat_id"
    PHOTO_MESSAGE_TAG = "photo_message"
    PHOTO_KEY = "photo"
    AUDIO_MESSAGE_TAG = "audio_message"
    AUDIO_KEY = "audio"
    VIDEO_MESSAGE_TAG = "video_message"
    VIDEO_KEY = "video"

    @get("setWebhook", public=True)
    def set_webhook(self, url: str, allowed_updates: List[str], drop_pending_updates: bool) -> str:
        File.create(self.client, blocks=[], tags=[Tag(kind=self.WEBHOOK_TAG, name=url)])
        return "OK"

    @get("sendMessage", public=True)
    def send_message(self, chat_id: int, text: str):
        File.create(
            self.client,
            blocks=[],
            tags=[Tag(kind=self.TEXT_MESSAGE_TAG, name=text, value={self.CHAT_ID_KEY: chat_id})],
        )
        return "OK"

    @post("sendPhoto", public=True)
    def send_photo(
        self, chat_id: int, photo: str
    ):  # realize str is wrong here, but seems to be what I get from the file upload to our server.
        File.create(
            self.client,
            blocks=[],
            tags=[
                Tag(
                    kind=self.PHOTO_MESSAGE_TAG,
                    value={self.CHAT_ID_KEY: chat_id, self.PHOTO_KEY: photo},
                )
            ],
        )
        return "OK"

    @post("sendAudio", public=True)
    def send_audio(
        self, chat_id: int, audio: str
    ):  # realize str is wrong here, but seems to be what I get from the file upload to our server.
        File.create(
            self.client,
            blocks=[],
            tags=[
                Tag(
                    kind=self.AUDIO_MESSAGE_TAG,
                    value={self.CHAT_ID_KEY: chat_id, self.AUDIO_KEY: audio},
                )
            ],
        )
        return "OK"

    @post("sendVideo", public=True)
    def send_video(
        self, chat_id: int, video: str
    ):  # realize str is wrong here, but seems to be what I get from the file upload to our server.
        File.create(
            self.client,
            blocks=[],
            tags=[
                Tag(
                    kind=self.VIDEO_MESSAGE_TAG,
                    value={self.CHAT_ID_KEY: chat_id, self.VIDEO_KEY: video},
                )
            ],
        )
        return "OK"
